@@ -27,7 +27,7 @@ and [Wu2016parallelkg]_.
 from __future__ import annotations
 
 from copy import deepcopy
-from typing import Any, Callable, Optional, Tuple, Union
+from typing import Any, Callable, Dict, Optional, Tuple, Type, Union
 
 import torch
 from botorch import settings
@@ -189,12 +189,12 @@ class qKnowledgeGradient(MCAcquisitionFunction, OneShotAcquisitionFunction):
 
     @concatenate_pending_points
     @t_batch_mode_transform()
-    def evaluate(self, X_actual: Tensor, bounds: Tensor, **kwargs: Any) -> Tensor:
+    def evaluate(self, X: Tensor, bounds: Tensor, **kwargs: Any) -> Tensor:
         r"""Evaluate qKnowledgeGradient on the candidate set `X_actual` by
         solving the inner optimization problem.
 
         Args:
-            X_actual: A `b x q x d` Tensor with `b` t-batches of `q` design points
+            X: A `b x q x d` Tensor with `b` t-batches of `q` design points
                 each. Unlike `forward()`, this does not include solutions of the
                 inner optimization problem.
             bounds: A `2 x d` tensor of lower and upper bounds for each column of
@@ -206,23 +206,30 @@ class qKnowledgeGradient(MCAcquisitionFunction, OneShotAcquisitionFunction):
 
         Returns:
             A Tensor of shape `b`. For t-batch b, the q-KG value of the design
-                `X_actual[b]` is averaged across the fantasy models.
+                `X[b]` is averaged across the fantasy models.
                 NOTE: If `current_value` is not provided, then this is not the
-                true KG value of `X_actual[b]`.
+                true KG value of `X[b]`.
         """
+        if hasattr(self, "expand"):
+            X = self.expand(X)
+
         # construct the fantasy model of shape `num_fantasies x b`
         fantasy_model = self.model.fantasize(
-            X=X_actual, sampler=self.sampler, observation_noise=True
+            X=X, sampler=self.sampler, observation_noise=True
         )
 
         # get the value function
         value_function = _get_value_function(
-            model=fantasy_model, objective=self.objective, sampler=self.inner_sampler
+            model=fantasy_model,
+            objective=self.objective,
+            sampler=self.inner_sampler,
+            project=getattr(self, "project", None),
         )
 
+        from botorch.generation.gen import gen_candidates_scipy
+
         # optimize the inner problem
         from botorch.optim.initializers import gen_value_function_initial_conditions
-        from botorch.generation.gen import gen_candidates_scipy
 
         initial_conditions = gen_value_function_initial_conditions(
             acq_function=value_function,
@@ -244,7 +251,11 @@ class qKnowledgeGradient(MCAcquisitionFunction, OneShotAcquisitionFunction):
         values, _ = torch.max(values, dim=0)
         if self.current_value is not None:
             values = values - self.current_value
-
+        # NOTE: using getattr to cover both no-attribute with qKG and None with qMFKG
+        if getattr(self, "cost_aware_utility", None) is not None:
+            values = self.cost_aware_utility(
+                X=X, deltas=values, sampler=self.cost_sampler
+            )
         # return average over the fantasy samples
         return values.mean(dim=0)
 
@@ -279,6 +290,9 @@ class qMultiFidelityKnowledgeGradient(qKnowledgeGradient):
     A version of `qKnowledgeGradient` that supports multi-fidelity optimization
     via a `CostAwareUtility` and the `project` and `expand` operators. If none
     of these are set, this acquisition function reduces to `qKnowledgeGradient`.
+    Through `valfunc_cls` and `valfunc_argfac`, this can be changed into a custom
+    multifidelity acquisition function (it is only KG if the terminal value is
+    computed using a posterior mean).
     """
 
     def __init__(
@@ -293,6 +307,8 @@ class qMultiFidelityKnowledgeGradient(qKnowledgeGradient):
         cost_aware_utility: Optional[CostAwareUtility] = None,
         project: Callable[[Tensor], Tensor] = lambda X: X,
         expand: Callable[[Tensor], Tensor] = lambda X: X,
+        valfunc_cls: Optional[Type[AcquisitionFunction]] = None,
+        valfunc_argfac: Optional[Callable[[Model, Dict[str, Any]]]] = None,
         **kwargs: Any,
     ) -> None:
         r"""Multi-Fidelity q-Knowledge Gradient (one-shot optimization).
@@ -320,13 +336,18 @@ class qMultiFidelityKnowledgeGradient(qKnowledgeGradient):
             cost_aware_utility: A CostAwareUtility computing the cost-transformed
                 utility from a candidate set and samples of increases in utility.
             project: A callable mapping a `batch_shape x q x d` tensor of design
-                points to a tensor of the same shape projected to the desired
-                target set (e.g. the target fidelities in case of multi-fidelity
-                optimization).
+                points to a tensor with shape `batch_shape x q_term x d` projected
+                to the desired target set (e.g. the target fidelities in case of
+                multi-fidelity optimization). For the basic case, `q_term = q`.
             expand: A callable mapping a `batch_shape x q x d` input tensor to
                 a `batch_shape x (q + q_e)' x d`-dim output tensor, where the
                 `q_e` additional points in each q-batch correspond to
                 additional ("trace") observations.
+            valfunc_cls: An acquisition function class to be used as the terminal
+                value function.
+            valfunc_argfac: An argument factory, i.e. callable that maps a `Model`
+                to a dictionary of kwargs for the terminal value function (e.g.
+                `best_f` for `ExpectedImprovement`).
         """
         if current_value is None and cost_aware_utility is not None:
             raise UnsupportedError(
@@ -345,6 +366,8 @@ class qMultiFidelityKnowledgeGradient(qKnowledgeGradient):
         self.project = project
         self.expand = expand
         self._cost_sampler = None
+        self.valfunc_cls = valfunc_cls
+        self.valfunc_argfac = valfunc_argfac
 
     @property
     def cost_sampler(self):
@@ -405,16 +428,20 @@ class qMultiFidelityKnowledgeGradient(qKnowledgeGradient):
         fantasy_model = self.model.fantasize(
             X=self.expand(X_eval), sampler=self.sampler, observation_noise=True
         )
-
         # get the value function
         value_function = _get_value_function(
-            model=fantasy_model, objective=self.objective, sampler=self.inner_sampler
+            model=fantasy_model,
+            objective=self.objective,
+            sampler=self.inner_sampler,
+            project=self.project,
+            valfunc_cls=self.valfunc_cls,
+            valfunc_argfac=self.valfunc_argfac,
         )
 
         # make sure to propagate gradients to the fantasy model train inputs
         # project the fantasy points
         with settings.propagate_grads(True):
-            values = value_function(X=self.project(X_fantasies))  # num_fantasies x b
+            values = value_function(X=X_fantasies)  # num_fantasies x b
 
         if self.current_value is not None:
             values = values - self.current_value
@@ -428,16 +455,57 @@ class qMultiFidelityKnowledgeGradient(qKnowledgeGradient):
         return values.mean(dim=0)
 
 
+class ProjectedAcquisitionFunction(AcquisitionFunction):
+    r"""
+    Defines a wrapper around  an `AcquisitionFunction` that incorporates the project
+    operator. Typically used to handle value functions in look-ahead methods.
+    """
+
+    def __init__(
+        self,
+        base_value_function: AcquisitionFunction,
+        project: Callable[[Tensor], Tensor],
+    ) -> None:
+        super().__init__(base_value_function.model)
+        self.base_value_function = base_value_function
+        self.project = project
+        self.objective = base_value_function.objective
+        self.sampler = getattr(base_value_function, "sampler", None)
+
+    def forward(self, X: Tensor) -> Tensor:
+        return self.base_value_function(self.project(X))
+
+
 def _get_value_function(
     model: Model,
     objective: Optional[Union[MCAcquisitionObjective, ScalarizedObjective]] = None,
     sampler: Optional[MCSampler] = None,
+    project: Optional[Callable[[Tensor], Tensor]] = None,
+    valfunc_cls: Optional[Type[AcquisitionFunction]] = None,
+    valfunc_argfac: Optional[Callable[[Model, Dict[str, Any]]]] = None,
 ) -> AcquisitionFunction:
     r"""Construct value function (i.e. inner acquisition function)."""
-    if isinstance(objective, MCAcquisitionObjective):
-        return qSimpleRegret(model=model, sampler=sampler, objective=objective)
+    if valfunc_cls is not None:
+        common_kwargs: Dict[str, Any] = {"model": model, "objective": objective}
+        if issubclass(valfunc_cls, MCAcquisitionFunction):
+            common_kwargs["sampler"] = sampler
+        kwargs = valfunc_argfac(model=model) if valfunc_argfac is not None else {}
+        base_value_function = valfunc_cls(**common_kwargs, **kwargs)
     else:
-        return PosteriorMean(model=model, objective=objective)
+        if isinstance(objective, MCAcquisitionObjective):
+            base_value_function = qSimpleRegret(
+                model=model, sampler=sampler, objective=objective
+            )
+        else:
+            base_value_function = PosteriorMean(model=model, objective=objective)
+
+    if project is None:
+        return base_value_function
+    else:
+        return ProjectedAcquisitionFunction(
+            base_value_function=base_value_function,
+            project=project,
+        )
 
 
 def _split_fantasy_points(X: Tensor, n_f: int) -> Tuple[Tensor, Tensor]:
