@@ -104,7 +104,7 @@ class qExpectedImprovement(MCAcquisitionFunction):
     Example:
         >>> model = SingleTaskGP(train_X, train_Y)
         >>> best_f = train_Y.max()[0]
-        >>> sampler = SobolQMCNormalSampler(1000)
+        >>> sampler = SobolQMCNormalSampler(1024)
         >>> qEI = qExpectedImprovement(model, best_f, sampler)
         >>> qei = qEI(test_X)
     """
@@ -126,8 +126,8 @@ class qExpectedImprovement(MCAcquisitionFunction):
                 a `batch_shape`-shaped tensor, which in case of a batched model
                 specifies potentially different values for each element of the batch.
             sampler: The sampler used to draw base samples. Defaults to
-                `SobolQMCNormalSampler(num_samples=500, collapse_batch_dims=True)`
-            objective: The MCAcquisitionObjective under which the samples are evalauted.
+                `SobolQMCNormalSampler(num_samples=512, collapse_batch_dims=True)`
+            objective: The MCAcquisitionObjective under which the samples are evaluated.
                 Defaults to `IdentityMCObjective()`.
             X_pending:  A `m x d`-dim Tensor of `m` design points that have been
                 submitted for function evaluation but have not yet been evaluated.
@@ -137,9 +137,7 @@ class qExpectedImprovement(MCAcquisitionFunction):
         super().__init__(
             model=model, sampler=sampler, objective=objective, X_pending=X_pending
         )
-        if not torch.is_tensor(best_f):
-            best_f = torch.tensor(float(best_f))
-        self.register_buffer("best_f", best_f)
+        self.register_buffer("best_f", torch.as_tensor(best_f, dtype=float))
 
     @concatenate_pending_points
     @t_batch_mode_transform()
@@ -157,8 +155,8 @@ class qExpectedImprovement(MCAcquisitionFunction):
         """
         posterior = self.model.posterior(X)
         samples = self.sampler(posterior)
-        obj = self.objective(samples)
-        obj = (obj - self.best_f.unsqueeze(-1)).clamp_min(0)
+        obj = self.objective(samples, X=X)
+        obj = (obj - self.best_f.unsqueeze(-1).to(obj)).clamp_min(0)
         q_ei = obj.max(dim=-1)[0].mean(dim=0)
         return q_ei
 
@@ -176,7 +174,7 @@ class qNoisyExpectedImprovement(MCAcquisitionFunction):
 
     Example:
         >>> model = SingleTaskGP(train_X, train_Y)
-        >>> sampler = SobolQMCNormalSampler(1000)
+        >>> sampler = SobolQMCNormalSampler(1024)
         >>> qNEI = qNoisyExpectedImprovement(model, train_X, sampler)
         >>> qnei = qNEI(test_X)
     """
@@ -199,7 +197,7 @@ class qNoisyExpectedImprovement(MCAcquisitionFunction):
                 that have already been observed. These points are considered as
                 the potential best design point.
             sampler: The sampler used to draw base samples. Defaults to
-                `SobolQMCNormalSampler(num_samples=500, collapse_batch_dims=True)`.
+                `SobolQMCNormalSampler(num_samples=512, collapse_batch_dims=True)`.
             objective: The MCAcquisitionObjective under which the samples are
                 evaluated. Defaults to `IdentityMCObjective()`.
             X_pending: A `batch_shape x m x d`-dim Tensor of `m` design points
@@ -218,7 +216,10 @@ class qNoisyExpectedImprovement(MCAcquisitionFunction):
         )
         if prune_baseline:
             X_baseline = prune_inferior_points(
-                model=model, X=X_baseline, objective=objective
+                model=model,
+                X=X_baseline,
+                objective=objective,
+                marginalize_dim=kwargs.get("marginalize_dim"),
             )
         self.register_buffer("X_baseline", X_baseline)
 
@@ -238,12 +239,12 @@ class qNoisyExpectedImprovement(MCAcquisitionFunction):
         """
         q = X.shape[-2]
         X_full = torch.cat([X, match_batch_shape(self.X_baseline, X)], dim=-2)
-        # TODO (T41248036): Implement more efficient way to compute posterior
-        # over both training and test points in GPyTorch
+        # TODO: Implement more efficient way to compute posterior over both training and
+        # test points in GPyTorch (https://github.com/cornellius-gp/gpytorch/issues/567)
         posterior = self.model.posterior(X_full)
         samples = self.sampler(posterior)
-        obj = self.objective(samples)
-        diffs = obj[:, :, :q].max(dim=-1)[0] - obj[:, :, q:].max(dim=-1)[0]
+        obj = self.objective(samples, X=X_full)
+        diffs = obj[..., :q].max(dim=-1).values - obj[..., q:].max(dim=-1).values
         return diffs.clamp_min(0).mean(dim=0)
 
 
@@ -261,7 +262,7 @@ class qProbabilityOfImprovement(MCAcquisitionFunction):
     Example:
         >>> model = SingleTaskGP(train_X, train_Y)
         >>> best_f = train_Y.max()[0]
-        >>> sampler = SobolQMCNormalSampler(1000)
+        >>> sampler = SobolQMCNormalSampler(1024)
         >>> qPI = qProbabilityOfImprovement(model, best_f, sampler)
         >>> qpi = qPI(test_X)
     """
@@ -283,7 +284,7 @@ class qProbabilityOfImprovement(MCAcquisitionFunction):
                 be a `batch_shape`-shaped tensor, which in case of a batched model
                 specifies potentially different values for each element of the batch.
             sampler: The sampler used to draw base samples. Defaults to
-                `SobolQMCNormalSampler(num_samples=500, collapse_batch_dims=True)`
+                `SobolQMCNormalSampler(num_samples=512, collapse_batch_dims=True)`
             objective: The MCAcquisitionObjective under which the samples are
                 evaluated. Defaults to `IdentityMCObjective()`.
             X_pending:  A `m x d`-dim Tensor of `m` design points that have
@@ -298,12 +299,8 @@ class qProbabilityOfImprovement(MCAcquisitionFunction):
         super().__init__(
             model=model, sampler=sampler, objective=objective, X_pending=X_pending
         )
-        if not torch.is_tensor(best_f):
-            best_f = torch.tensor(float(best_f))
-        self.register_buffer("best_f", best_f)
-        if not torch.is_tensor(tau):
-            tau = torch.tensor(float(tau))
-        self.register_buffer("tau", tau)
+        self.register_buffer("best_f", torch.as_tensor(best_f, dtype=float))
+        self.register_buffer("tau", torch.as_tensor(tau, dtype=float))
 
     @concatenate_pending_points
     @t_batch_mode_transform()
@@ -321,9 +318,9 @@ class qProbabilityOfImprovement(MCAcquisitionFunction):
         """
         posterior = self.model.posterior(X)
         samples = self.sampler(posterior)
-        obj = self.objective(samples)
+        obj = self.objective(samples, X=X)
         max_obj = obj.max(dim=-1)[0]
-        impr = max_obj - self.best_f.unsqueeze(-1)
+        impr = max_obj - self.best_f.unsqueeze(-1).to(max_obj)
         val = torch.sigmoid(impr / self.tau).mean(dim=0)
         return val
 
@@ -337,7 +334,7 @@ class qSimpleRegret(MCAcquisitionFunction):
 
     Example:
         >>> model = SingleTaskGP(train_X, train_Y)
-        >>> sampler = SobolQMCNormalSampler(1000)
+        >>> sampler = SobolQMCNormalSampler(1024)
         >>> qSR = qSimpleRegret(model, sampler)
         >>> qsr = qSR(test_X)
     """
@@ -358,7 +355,7 @@ class qSimpleRegret(MCAcquisitionFunction):
         """
         posterior = self.model.posterior(X)
         samples = self.sampler(posterior)
-        obj = self.objective(samples)
+        obj = self.objective(samples, X=X)
         val = obj.max(dim=-1)[0].mean(dim=0)
         return val
 
@@ -374,7 +371,7 @@ class qUpperConfidenceBound(MCAcquisitionFunction):
 
     Example:
         >>> model = SingleTaskGP(train_X, train_Y)
-        >>> sampler = SobolQMCNormalSampler(1000)
+        >>> sampler = SobolQMCNormalSampler(1024)
         >>> qUCB = qUpperConfidenceBound(model, 0.1, sampler)
         >>> qucb = qUCB(test_X)
     """
@@ -393,7 +390,7 @@ class qUpperConfidenceBound(MCAcquisitionFunction):
             model: A fitted model.
             beta: Controls tradeoff between mean and standard deviation in UCB.
             sampler: The sampler used to draw base samples. Defaults to
-                `SobolQMCNormalSampler(num_samples=500, collapse_batch_dims=True)`
+                `SobolQMCNormalSampler(num_samples=512, collapse_batch_dims=True)`
             objective: The MCAcquisitionObjective under which the samples are
                 evaluated. Defaults to `IdentityMCObjective()`.
             X_pending: A `batch_shape x m x d`-dim Tensor of `m` design points that have
@@ -422,7 +419,7 @@ class qUpperConfidenceBound(MCAcquisitionFunction):
         """
         posterior = self.model.posterior(X)
         samples = self.sampler(posterior)
-        obj = self.objective(samples)
+        obj = self.objective(samples, X=X)
         mean = obj.mean(dim=0)
         ucb_samples = mean + self.beta_prime * (obj - mean).abs()
         return ucb_samples.max(dim=-1)[0].mean(dim=0)
