@@ -4,14 +4,14 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-from .errors import (
+from botorch.exceptions.errors import (
     BotorchError,
     BotorchTensorDimensionError,
     CandidateGenerationError,
     InputDataError,
     UnsupportedError,
 )
-from .warnings import (
+from botorch.exceptions.warnings import (
     BadInitialCandidatesWarning,
     BotorchTensorDimensionWarning,
     BotorchWarning,
