@@ -4,6 +4,10 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+from botorch.models.approximate_gp import (
+    ApproximateGPyTorchModel,
+    SingleTaskVariationalGP,
+)
 from botorch.models.cost import AffineFidelityCostModel
 from botorch.models.deterministic import (
     AffineDeterministicModel,
@@ -17,6 +21,7 @@ from botorch.models.gp_regression import (
 from botorch.models.gp_regression_fidelity import SingleTaskMultiFidelityGP
 from botorch.models.gp_regression_mixed import MixedSingleTaskGP
 from botorch.models.higher_order_gp import HigherOrderGP
+from botorch.models.model import ModelList
 from botorch.models.model_list_gp_regression import ModelListGP
 from botorch.models.multitask import (
     FixedNoiseMultiTaskGP,
@@ -28,6 +33,7 @@ from botorch.models.pairwise_gp import PairwiseGP, PairwiseLaplaceMarginalLogLik
 __all__ = [
     "AffineDeterministicModel",
     "AffineFidelityCostModel",
+    "ApproximateGPyTorchModel",
     "FixedNoiseGP",
     "FixedNoiseMultiTaskGP",
     "GenericDeterministicModel",
@@ -35,10 +41,12 @@ __all__ = [
     "HigherOrderGP",
     "KroneckerMultiTaskGP",
     "MixedSingleTaskGP",
+    "ModelList",
     "ModelListGP",
     "MultiTaskGP",
     "PairwiseGP",
     "PairwiseLaplaceMarginalLogLikelihood",
     "SingleTaskGP",
     "SingleTaskMultiFidelityGP",
+    "SingleTaskVariationalGP",
 ]
