@@ -10,6 +10,8 @@ from botorch.optim.optimize import (
     gen_batch_initial_conditions,
     optimize_acqf,
     optimize_acqf_cyclic,
+    optimize_acqf_discrete,
+    optimize_acqf_mixed,
 )
 from botorch.optim.stopping import ExpMAStoppingCriterion
 
@@ -20,6 +22,8 @@ __all__ = [
     "initialize_q_batch_nonneg",
     "optimize_acqf",
     "optimize_acqf_cyclic",
+    "optimize_acqf_discrete",
+    "optimize_acqf_mixed",
     "module_to_array",
     "set_params_with_array",
     "ExpMAStoppingCriterion",
