@@ -4,9 +4,18 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
+r"""
+References
+
+.. [Regis]
+    R. G. Regis, C. A. Shoemaker. Combining radial basis function
+    surrogates and dynamic coordinate search in high-dimensional
+    expensive black-box optimization, Engineering Optimization, 2013.
+"""
 from __future__ import annotations
 
 import warnings
+from math import ceil
 from typing import Dict, List, Optional, Tuple, Union
 
 import torch
@@ -17,17 +26,24 @@ from botorch.acquisition.knowledge_gradient import (
     qKnowledgeGradient,
 )
 from botorch.acquisition.utils import is_nonnegative
-from botorch.exceptions.warnings import BadInitialCandidatesWarning, SamplingWarning
+from botorch.exceptions.errors import BotorchTensorDimensionError
+from botorch.exceptions.warnings import (
+    BotorchWarning,
+    BadInitialCandidatesWarning,
+    SamplingWarning,
+)
 from botorch.models.model import Model
-from botorch.optim.utils import fix_features
+from botorch.optim.utils import fix_features, get_X_baseline
+from botorch.utils.multi_objective.pareto import is_non_dominated
 from botorch.utils.sampling import (
     batched_multinomial,
     draw_sobol_samples,
     manual_seed,
     get_polytope_samples,
 )
-from botorch.utils.transforms import standardize
+from botorch.utils.transforms import unnormalize, normalize, standardize
 from torch import Tensor
+from torch.distributions import Normal
 from torch.quasirandom import SobolEngine
 
 
@@ -53,7 +69,8 @@ def gen_batch_initial_conditions(
         num_restarts: The number of starting points for multistart acquisition
             function optimization.
         raw_samples: The number of raw samples to consider in the initialization
-            heuristic.
+            heuristic. Note: if `sample_around_best` is True (the default is False),
+            then `2 * raw_samples` samples are used.
         fixed_features: A map `{feature_index: value}` for features that
             should be fixed to a particular value during generation.
         options: Options for initial condition generation. For valid options see
@@ -137,7 +154,24 @@ def gen_batch_initial_conditions(
                     .view(n, q, -1)
                     .cpu()
                 )
-
+            # sample points around best
+            if options.get("sample_around_best", False):
+                X_best_rnd = sample_points_around_best(
+                    acq_function=acq_function,
+                    n_discrete_points=n * q,
+                    sigma=options.get("sample_around_best_sigma", 1e-3),
+                    bounds=bounds,
+                    subset_sigma=options.get("sample_around_best_subset_sigma", 1e-1),
+                    prob_perturb=options.get("sample_around_best_prob_perturb"),
+                )
+                if X_best_rnd is not None:
+                    X_rnd = torch.cat(
+                        [
+                            X_rnd,
+                            X_best_rnd.view(n, q, bounds.shape[-1]).cpu(),
+                        ],
+                        dim=0,
+                    )
             X_rnd = fix_features(X_rnd, fixed_features=fixed_features)
             with torch.no_grad():
                 if batch_limit is None:
@@ -566,3 +600,234 @@ def initialize_q_batch_nonneg(
     if max_idx not in idcs:
         idcs[-1] = max_idx
     return X[idcs]
+
+
+def sample_points_around_best(
+    acq_function: AcquisitionFunction,
+    n_discrete_points: int,
+    sigma: float,
+    bounds: Tensor,
+    best_pct: float = 5.0,
+    subset_sigma: float = 1e-1,
+    prob_perturb: Optional[float] = None,
+) -> Optional[Tensor]:
+    r"""Find best points and sample nearby points.
+
+    Args:
+        acq_function: The acquisition function.
+        n_discrete_points: The number of points to sample.
+        sigma: The standard deviation of the additive gaussian noise for
+            perturbing the best points.
+        bounds: A `2 x d`-dim tensor containing the bounds.
+        best_pct: The percentage of best points to perturb.
+        subset_sigma: The standard deviation of the additive gaussian
+            noise for perturbing a subset of dimensions of the best points.
+        prob_perturb: The probability of perturbing each dimension.
+
+    Returns:
+        An optional `n_discrete_points x d`-dim tensor containing the
+            sampled points. This is None if no baseline points are found.
+    """
+    X = get_X_baseline(acq_function=acq_function)
+    if X is None:
+        return
+    with torch.no_grad():
+        try:
+            posterior = acq_function.model.posterior(X)
+        except AttributeError:
+            warnings.warn(
+                "Failed to sample around previous best points.",
+                BotorchWarning,
+            )
+            return
+        mean = posterior.mean
+        while mean.ndim > 2:
+            # take average over batch dims
+            mean = mean.mean(dim=0)
+        try:
+            f_pred = acq_function.objective(mean)
+        # Some acquisition functions do not have an objective
+        # and for some acquisition functions the objective is None
+        except (AttributeError, TypeError):
+            f_pred = mean
+        if hasattr(acq_function, "maximize"):
+            # make sure that the optimiztaion direction is set properly
+            if not acq_function.maximize:
+                f_pred = -f_pred
+        try:
+            # handle constraints for EHVI-based acquisition functions
+            constraints = acq_function.constraints
+            if constraints is not None:
+                neg_violation = -torch.stack(
+                    [c(mean).clamp_min(0.0) for c in constraints], dim=-1
+                ).sum(dim=-1)
+                feas = neg_violation == 0
+                if feas.any():
+                    f_pred[~feas] = float("-inf")
+                else:
+                    # set objective equal to negative violation
+                    f_pred = neg_violation
+        except AttributeError:
+            pass
+        if f_pred.ndim == mean.ndim and f_pred.shape[-1] > 1:
+            # multi-objective
+            # find pareto set
+            is_pareto = is_non_dominated(f_pred)
+            best_X = X[is_pareto]
+        else:
+            if f_pred.shape[-1] == 1:
+                f_pred = f_pred.squeeze(-1)
+            n_best = max(1, round(X.shape[0] * best_pct / 100))
+            # the view() is to ensure that best_idcs is not a scalar tensor
+            best_idcs = torch.topk(f_pred, n_best).indices.view(-1)
+            best_X = X[best_idcs]
+    n_trunc_normal_points = (
+        n_discrete_points // 2 if best_X.shape[-1] >= 20 else n_discrete_points
+    )
+    perturbed_X = sample_truncated_normal_perturbations(
+        X=best_X,
+        n_discrete_points=n_trunc_normal_points,
+        sigma=sigma,
+        bounds=bounds,
+    )
+    if best_X.shape[-1] > 20 or prob_perturb is not None:
+        perturbed_subset_dims_X = sample_perturbed_subset_dims(
+            X=best_X,
+            bounds=bounds,
+            # ensure that we return n_discrete_points
+            n_discrete_points=n_discrete_points - n_trunc_normal_points,
+            sigma=sigma,
+            prob_perturb=prob_perturb,
+        )
+        perturbed_X = torch.cat([perturbed_X, perturbed_subset_dims_X], dim=0)
+        # shuffle points
+        perm = torch.randperm(perturbed_X.shape[0], device=X.device)
+        perturbed_X = perturbed_X[perm]
+    return perturbed_X
+
+
+def sample_truncated_normal_perturbations(
+    X: Tensor,
+    n_discrete_points: int,
+    sigma: float,
+    bounds: Tensor,
+    qmc: bool = True,
+) -> Tensor:
+    r"""Sample points around `X`.
+
+    Sample perturbed points around `X` such that the added perturbations
+    are sampled from N(0, sigma^2 I) and truncated to be within [0,1]^d.
+
+    Args:
+        X: A `n x d`-dim tensor starting points.
+        n_discrete_points: The number of points to sample.
+        sigma: The standard deviation of the additive gaussian noise for
+            perturbing the points.
+        bounds: A `2 x d`-dim tensor containing the bounds.
+        qmc: A boolean indicating whether to use qmc.
+
+    Returns:
+        A `n_discrete_points x d`-dim tensor containing the sampled points.
+    """
+    X = normalize(X, bounds=bounds)
+    d = X.shape[1]
+    # sample points from N(X_center, sigma^2 I), truncated to be within
+    # [0, 1]^d.
+    if X.shape[0] > 1:
+        rand_indices = torch.randint(X.shape[0], (n_discrete_points,), device=X.device)
+        X = X[rand_indices]
+    if qmc:
+        std_bounds = torch.zeros(2, d, dtype=X.dtype, device=X.device)
+        std_bounds[1] = 1
+        u = draw_sobol_samples(bounds=std_bounds, n=n_discrete_points, q=1).squeeze(1)
+    else:
+        u = torch.rand((n_discrete_points, d), dtype=X.dtype, device=X.device)
+    # compute bounds to sample from
+    a = -X
+    b = 1 - X
+    # compute z-score of bounds
+    alpha = a / sigma
+    beta = b / sigma
+    normal = Normal(0, 1)
+    cdf_alpha = normal.cdf(alpha)
+    # use inverse transform
+    perturbation = normal.icdf(cdf_alpha + u * (normal.cdf(beta) - cdf_alpha)) * sigma
+    # add perturbation and clip points that are still outside
+    perturbed_X = (X + perturbation).clamp(0.0, 1.0)
+    return unnormalize(perturbed_X, bounds=bounds)
+
+
+def sample_perturbed_subset_dims(
+    X: Tensor,
+    bounds: Tensor,
+    n_discrete_points: int,
+    sigma: float = 1e-1,
+    qmc: bool = True,
+    prob_perturb: Optional[float] = None,
+) -> Tensor:
+    r"""Sample around `X` by perturbing a subset of the dimensions.
+
+    By default, dimensions are perturbed with probability equal to
+    `min(20 / d, 1)`. As shown in [Regis]_, perturbing a small number
+    of dimensions can be beneificial. The perturbations are sampled
+    from N(0, sigma^2 I) and truncated to be within [0,1]^d.
+
+    Args:
+        X: A `n x d`-dim tensor starting points. `X`
+            must be normalized to be within `[0, 1]^d`.
+        bounds: The bounds to sample perturbed values from
+        n_discrete_points: The number of points to sample.
+        sigma: The standard deviation of the additive gaussian noise for
+            perturbing the points.
+        qmc: A boolean indicating whether to use qmc.
+        prob_perturb: The probability of perturbing each dimension. If omitted,
+            defaults to `min(20 / d, 1)`.
+
+    Returns:
+        A `n_discrete_points x d`-dim tensor containing the sampled points.
+
+    """
+    if bounds.ndim != 2:
+        raise BotorchTensorDimensionError("bounds must be a `2 x d`-dim tensor.")
+    elif X.ndim != 2:
+        raise BotorchTensorDimensionError("X must be a `n x d`-dim tensor.")
+    d = bounds.shape[-1]
+    if prob_perturb is None:
+        # Only perturb a subset of the features
+        prob_perturb = min(20.0 / d, 1.0)
+
+    if X.shape[0] == 1:
+        X_cand = X.repeat(n_discrete_points, 1)
+    else:
+        rand_indices = torch.randint(X.shape[0], (n_discrete_points,), device=X.device)
+        X_cand = X[rand_indices]
+    pert = sample_truncated_normal_perturbations(
+        X=X_cand,
+        n_discrete_points=n_discrete_points,
+        sigma=sigma,
+        bounds=bounds,
+        qmc=qmc,
+    )
+
+    # find cases where we are not perturbing any dimensions
+    mask = (
+        torch.rand(
+            n_discrete_points,
+            d,
+            dtype=bounds.dtype,
+            device=bounds.device,
+        )
+        <= prob_perturb
+    )
+    ind = (~mask).all(dim=-1).nonzero()
+    # perturb `n_perturb` of the dimensions
+    n_perturb = ceil(d * prob_perturb)
+    perturb_mask = torch.zeros(d, dtype=mask.dtype, device=mask.device)
+    perturb_mask[:n_perturb].fill_(1)
+    # TODO: use batched `torch.randperm` when available:
+    # https://github.com/pytorch/pytorch/issues/42502
+    for idx in ind:
+        mask[idx] = perturb_mask[torch.randperm(d, device=bounds.device)]
+    # Create candidate points
+    X_cand[mask] = pert[mask]
+    return X_cand
