@@ -8,7 +8,7 @@ from botorch.posteriors.deterministic import DeterministicPosterior
 from botorch.posteriors.gpytorch import GPyTorchPosterior
 from botorch.posteriors.higher_order import HigherOrderGPPosterior
 from botorch.posteriors.multitask import MultitaskGPPosterior
-from botorch.posteriors.posterior import Posterior
+from botorch.posteriors.posterior import Posterior, PosteriorList
 from botorch.posteriors.transformed import TransformedPosterior
 
 
@@ -18,5 +18,6 @@ __all__ = [
     "HigherOrderGPPosterior",
     "MultitaskGPPosterior",
     "Posterior",
+    "PosteriorList",
     "TransformedPosterior",
 ]
