@@ -12,9 +12,14 @@ from botorch.test_functions.multi_fidelity import (
 from botorch.test_functions.multi_objective import (
     BNH,
     C2DTLZ2,
+    CarSideImpact,
     CONSTR,
     DTLZ1,
     DTLZ2,
+    DTLZ3,
+    DTLZ4,
+    DTLZ5,
+    DTLZ7,
     OSY,
     SRN,
     ZDT1,
@@ -22,6 +27,7 @@ from botorch.test_functions.multi_objective import (
     ZDT3,
     BraninCurrin,
     ConstrainedBraninCurrin,
+    WeldedBeam,
     VehicleSafety,
 )
 from botorch.test_functions.synthetic import (
@@ -61,12 +67,17 @@ __all__ = [
     "Bukin",
     "CONSTR",
     "Cosine8",
+    "CarSideImpact",
     "ConstrainedBraninCurrin",
     "C2DTLZ2",
     "DixonPrice",
     "DropWave",
     "DTLZ1",
     "DTLZ2",
+    "DTLZ3",
+    "DTLZ4",
+    "DTLZ5",
+    "DTLZ7",
     "EggHolder",
     "Griewank",
     "Hartmann",
@@ -84,6 +95,7 @@ __all__ = [
     "SyntheticTestFunction",
     "ThreeHumpCamel",
     "VehicleSafety",
+    "WeldedBeam",
     "ZDT1",
     "ZDT2",
     "ZDT3",
