@@ -7,6 +7,7 @@
 from botorch.utils.constraints import get_outcome_constraint_transforms
 from botorch.utils.feasible_volume import estimate_feasible_volume
 from botorch.utils.objective import apply_constraints, get_objective_weights_transform
+from botorch.utils.rounding import approximate_round
 from botorch.utils.sampling import (
     batched_multinomial,
     construct_base_samples,
@@ -24,16 +25,17 @@ from botorch.utils.transforms import (
 
 __all__ = [
     "apply_constraints",
+    "approximate_round",
     "batched_multinomial",
     "construct_base_samples",
     "construct_base_samples_from_posterior",
     "draw_sobol_normal_samples",
     "draw_sobol_samples",
+    "estimate_feasible_volume",
     "get_objective_weights_transform",
     "get_outcome_constraint_transforms",
     "manual_seed",
     "squeeze_last_dim",
     "standardize",
     "t_batch_mode_transform",
-    "estimate_feasible_volume",
 ]
