@@ -4,15 +4,14 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-from botorch.utils.multi_objective.box_decomposition import NondominatedPartitioning
-from botorch.utils.multi_objective.hypervolume import Hypervolume
+from botorch.utils.multi_objective.hypervolume import Hypervolume, infer_reference_point
 from botorch.utils.multi_objective.pareto import is_non_dominated
 from botorch.utils.multi_objective.scalarization import get_chebyshev_scalarization
 
 
 __all__ = [
     "get_chebyshev_scalarization",
+    "infer_reference_point",
     "is_non_dominated",
     "Hypervolume",
-    "NondominatedPartitioning",
 ]
